/*
	Phoenix Suite 2
	Author: Jean-Baptiste Wautier
	Copyright 2006-2009 (c)
*/
﻿namespace Phoenix.Log.Writer
{
    using System;

    /// <summary>
    /// Provides instanciation features for Logger.
    /// Configure the Logger with default configuration.
    /// SQLiteAppender is configured. The database is 'Log.db' in the application directory
    /// ConsoleAppender is configured.
    /// Both the appenders log at level 6 (Log everything) and are activated.
    /// </summary>
    public class DefaultConfigurator : Configurator, IConfigurator
    {
        #region Fields

        private ushort logLevel = 6;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DefaultConfigurator"/> class.
        /// </summary>
        public DefaultConfigurator()
            : base()
        {
            this.LogLevel = 6;
            this.Path = null;
            this.SilentError = true;
            this.LogIntoConsole = true;
            this.LogIntoDB = false;
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets a value indicating whether log in console.
        /// </summary>
        /// <value><c>true</c> if [log in console]; otherwise, <c>false</c>.</value>
        public bool LogIntoConsole
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether log in the SQLite database.
        /// </summary>
        /// <value><c>true</c> if [log in db]; otherwise, <c>false</c>.</value>
        public bool LogIntoDB
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the level of logging (Should be a value between 1 and 6).
        /// 1- Log only FATAL
        /// 2- Log ERROR and above
        /// 3- Log WARNING and above
        /// 4- Log INFO and above
        /// 5- Log DEBUG and above
        /// 6- Log FLAG and above (log ALL)
        /// </summary>
        /// <remarks>
        /// Any values bigger than 6 will be rounded to 6.
        /// Any values below or equal to 0 will be rounded to 1.
        /// </remarks>
        /// <value>The log level.</value>
        public ushort LogLevel
        {
            get { return this.logLevel; }
            set
            {
                if (value <= 0)
                    this.logLevel = 0;
                else if (value > 6)
                    this.logLevel = 6;
                else
                    this.logLevel = value;
            }
        }

        /// <summary>
        /// Gets or sets the errorPath of the log DB.
        /// </summary>
        /// <value>The errorPath.</value>
        public string Path
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets a value indicating whether an error should be rethrown or silently
        /// logged into a textWriter.
        /// </summary>
        /// <value><c>true</c> if [silent error]; otherwise, <c>false</c>.</value>
        public bool SilentError
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Gets an instance of the appender. This appender is configured from the sqliteConfig textWriter
        /// </summary>
        /// <param name="typeOfSender">The type of sender.</param>
        /// <returns>An instance of assembly appender</returns>
        public ILogger GetLogger(System.Type typeOfSender)
        {
            #region SQLite appender
            if (this.LogIntoDB)
            {
                IPersistor persistor = (this.Path == null)
            ? new SQLitePersistor()
            : new SQLitePersistor(this.Path);

                SQLiteAppender sqliteAppender = new SQLiteAppender(persistor);
                sqliteAppender.LogLevel = this.LogLevel;
                sqliteAppender.SilentError = this.SilentError;
                base.Logger.AddAppender(sqliteAppender);
            }
            #endregion

            #region Console appender
            if (this.LogIntoConsole)
            {
                ConsoleAppender consoleAppender = new ConsoleAppender();
                consoleAppender.LogLevel = this.LogLevel;
                consoleAppender.SilentError = this.SilentError;
                base.Logger.AddAppender(consoleAppender);
            }
            #endregion

            base.Logger.SetTypeOfSender(typeOfSender);
            return base.Logger;
        }

        #endregion Methods
    }
}